# Generated by Plain 0.21.5 on 2025-02-17 04:24

from plain import models
from plain.models import migrations


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="User",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True)),
                ("email", models.EmailField(max_length=254)),
                ("username", models.CharField(max_length=100)),
            ],
        ),
    ]
