# Generated by Plain 0.3.0 on 2024-08-16 16:46

from plain import models
from plain.models import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("users", "__first__"),
        ("plainoauth", "0004_alter_oauthconnection_access_token_and_more"),
    ]

    operations = [
        migrations.AddConstraint(
            model_name="oauthconnection",
            constraint=models.UniqueConstraint(
                fields=("provider_key", "provider_user_id"),
                name="unique_oauth_provider_user_id",
            ),
        ),
    ]
