# Generated by Plain 0.31.0 on 2025-03-08 21:33

from plain import models
from plain.models import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("plainoauth", "0005_alter_oauthconnection_unique_together_and_more"),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name="oauthconnection",
            name="unique_oauth_provider_user_id",
        ),
        migrations.AddConstraint(
            model_name="oauthconnection",
            constraint=models.UniqueConstraint(
                fields=("provider_key", "provider_user_id"),
                name="plainoauth_oauthconnection_unique_provider_key_user_id",
            ),
        ),
    ]
