# Generated by Plain 0.32.0 on 2025-03-10 15:37

from plain import models
from plain.models import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "plainoauth",
            "0006_remove_oauthconnection_unique_oauth_provider_user_id_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="oauthconnection",
            name="provider_key",
            field=models.CharField(max_length=100),
        ),
        migrations.AlterField(
            model_name="oauthconnection",
            name="provider_user_id",
            field=models.CharField(max_length=100),
        ),
    ]
